//
// cyc2.v
//
// cycloneII for Terasic DE1
//

module cyc2(
	clk24_0,
	clk24_1,
	clk27_0,
	clk27_1,
	clk50,
	ext_clk,
	flash_addr,
	flash_data,
	flash_rst,
	flash_ce,
	flash_oe,
	flash_we,
	sram_addr,
	sram_data,
	sram_ce,
	sram_oe,
	sram_we,
	sram_ub,
	sram_lb,
	dram_addr,
	dram_data,
	dram_clk,
	dram_cke,
	dram_cs,
	dram_we,
	dram_ba1,
	dram_ba0,
	dram_ras,
	dram_cas,
	dram_udqm,
	dram_ldqm,
	ps2_clk,
	ps2_dat,
	uart_txd,
	uart_rxd,
	ac_mclk,
	ac_bclk,
	ac_adlrck,
	ac_addat,
	ac_dalrck,
	ac_dadat,
	i2c_scl,
	i2c_sda,
	vga_r,
	vga_g,
	vga_b,
	vga_hsync,
	vga_vsync,
	sd_clk,
	sd_cmd,
	sd_dat,
	sd_dat3,
	hex0,
	hex1,
	hex2,
	hex3,
	ledr,
	ledg,
	sw,
	key,
	gpio0,
	gpio1,
	tck,
	tcs,
	tdi,
	tdo
);

// clock
input	clk24_0;	// 24MHz from USB Blaster
input	clk24_1;	// 24MHz from USB Blaster
input	clk27_0;	// 27MHz
input	clk27_1;	// 27MHz
input	clk50;	// 50MHz
input	ext_clk;	// external clock

// memory interface
// Flash
output [21:0]	flash_addr;
inout [7:0]		flash_data;
output	flash_rst;
output	flash_ce;
output	flash_oe;
output	flash_we;
// SRAM
output [17:0]	sram_addr;
inout [15:0]	sram_data;
output	sram_ce;
output	sram_oe;
output	sram_we;
output	sram_ub;
output	sram_lb;
// SDRAM
output [11:0]	dram_addr;
inout [15:0]	dram_data;
output	dram_clk;
output	dram_cke;
output	dram_cs;
output	dram_we;
output	dram_ba1;
output	dram_ba0;
output	dram_ras;
output	dram_cas;
output	dram_udqm;
output	dram_ldqm;

// PS/2 KB/Mouse interface
input	ps2_clk;
input	ps2_dat;

// UART interface
output	uart_txd;
input	uart_rxd;

// Audio Codec interface
output	ac_mclk;	// master clock
output	ac_bclk;	// bit clock (inout ???)
output	ac_adlrck;
input	ac_addat;
output	ac_dalrck;
output	ac_dadat;
output	i2c_scl;
inout	i2c_sda;

// VGA interface
output [3:0]	vga_r;
output [3:0]	vga_g;
output [3:0]	vga_b;
output	vga_hsync;
output	vga_vsync;

// SD Card interface
output	sd_clk;
inout	sd_cmd;
inout	sd_dat;
inout	sd_dat3;

// 7segment LED
output [6:0]	hex0;	// L : on
output [6:0]	hex1;	// L : on
output [6:0]	hex2;	// L : on
output [6:0]	hex3;	// L : on

// LED
output [9:0]	ledr;	// H : on
output [7:0]	ledg;	// H : on

// Slide Switch
input [9:0]		sw;		// Down : L, Up : H

// Key Switch
input [3:0]		key;	// pressed : L, released : H

// GPIO
inout [35:0]	gpio0;
inout [35:0]	gpio1;

// JTAG interface
input	tck;
input	tcs;
input	tdi;
output	tdo;

//
// default signal definitions
//

// memory interface
// Flash
assign	flash_addr = 22'h00_0000;
assign	flash_data = 8'hzz;
assign	flash_rst = 1'b1;
assign	flash_ce = 1'b1;
assign	flash_oe = 1'b1;
assign	flash_we = 1'b1;
// SRAM
//assign	sram_data = 16'hzzzz;
//assign	sram_ce = 1'b1;
//assign	sram_oe = 1'b1;
//assign	sram_we = 1'b1;
//assign	sram_ub = 1'b1;
//assign	sram_lb = 1'b1;
// SDRAM
assign	dram_clk = 1'b0;
assign	dram_cke = 1'b0;
assign	dram_addr = 12'h000;
assign	dram_data = 16'hzzzz;
assign	dram_cs = 1'b1;
assign	dram_we = 1'b1;
assign	dram_ras = 1'b1;
assign	dram_cas = 1'b1;
assign	dram_ba0 = 1'b0;
assign	dram_ba1 = 1'b0;
assign	dram_udqm = 1'b0;
assign	dram_ldqm = 1'b0;

// PS/2 KB/Mouse interface
//assign	ps2_dat = 1'bz;

// UART interface
//assign	uart_txd = 1'b1;

// Audio Codec interface
assign	ac_mclk = 1'b1;
assign	ac_bclk = 1'b1;
assign	ac_adlrck = 1'b1;
assign	ac_dalrck = 1'b1;
assign	ac_dadat = 1'b1;
assign	i2c_scl = 1'b1;
assign	i2c_sda = 1'bz;

// VGA interface
assign	vga_r = 4'b0000;
assign	vga_g = 4'b0000;
assign	vga_b = 4'b0000;
assign	vga_hsync = 1'b1;
assign	vga_vsync = 1'b1;

// SD Card interface
assign	sd_clk = 1'b1;
assign	sd_cmd = 1'bz;
assign	sd_dat = 1'bz;

// 7segment LED
assign	hex0 = 7'b111_1111;	// L : on
assign	hex1 = 7'b111_1111;	// L : on
assign	hex2 = 7'b111_1111;	// L : on
assign	hex3 = 7'b111_1111;	// L : on

// LED
assign	ledr = 10'b00_0000_0000;	// H : on
assign	ledg = 8'b0000_0000;	// H : on

// GPIO
assign	gpio0 = 36'hzzzzzzzzz;
assign	gpio1 = 36'hzzzzzzzzz;

// JTAG interface


wire	clk;
wire	locked;
wire	async_rst;
reg	rst;
wire [15:0]	addr;
wire	en;
wire	wen;
wire [7:0]	wdata;
wire [7:0]	rdata;
reg	ack;
wire [15:0]	ram_addr;
wire	ram_en;
wire	ram_wen;
wire [7:0]	ram_wdata;
wire [7:0]	ram_rdata;
wire [10:0]	rom0_addr;
wire	rom0_en;
wire [7:0]	rom0_rdata;
wire [10:0]	rom1_addr;
wire	rom1_en;
wire [7:0]	rom1_rdata;
wire	acia_addr;
wire	acia_en;
wire	acia_wen;
wire [7:0]	acia_wdata;
wire [7:0]	acia_rdata;

pll11 pll11_0(
	.inclk0(clk27_0),
	.c0(clk),
	.locked(locked)
);

assign	async_rst = key[0] & locked;

always @(posedge clk) begin
	rst <= async_rst;
end

ac09_top ac09_top0(
	.clk(clk),
	.rst(rst),
	.addr(addr),
	.en(en),
	.wen(wen),
	.wdata(wdata),
	.rdata(rdata),
	.ack(ack),
	.irq(irq),
	.fiq(fiq),
	.nmi(nmi)
);

always @(posedge clk or negedge rst) begin
	if(~rst)
		ack <= 1'b0;
	else
		ack <= en;
end

ast09_mux mux0(
	.clk(clk),
	.rst(rst),
	.addr(addr),
	.en(en),
	.wen(wen),
	.wdata(wdata),
	.rdata(rdata),
	.ram_addr(ram_addr),
	.ram_en(ram_en),
	.ram_wen(ram_wen),
	.ram_wdata(ram_wdata),
	.ram_rdata(ram_rdata),
	.rom0_addr(rom0_addr),
	.rom0_en(rom0_en),
	.rom0_rdata(rom0_rdata),
	.rom1_addr(rom1_addr),
	.rom1_en(rom1_en),
	.rom1_rdata(rom1_rdata),
	.acia_addr(acia_addr),
	.acia_en(acia_en),
	.acia_wen(acia_wen),
	.acia_rdata(acia_rdata),
	.acia_wdata(acia_wdata)
);

assign	sram_addr = {2'b00, ram_addr};
assign	sram_data = ram_wen ? {8'h00, ram_wdata} : 16'hzzzz;
assign	ram_rdata = sram_data[7:0];
assign	sram_ce = ~ram_en;
assign	sram_oe = ram_wen;
assign	sram_we = ~ram_wen | clk;
assign	sram_ub = 1'b1;
assign	sram_lb = 1'b0;

srom8 rom0(
	.address(rom0_addr),
	.clock(clk),
	.q(rom0_rdata)
);

assign	rom1_rdata = 8'h00;

acia #(16'h0905,16'h0090) acia0(
	.clk(clk),
	.rst(rst),
	.addr(acia_addr),
	.en(acia_en),
	.wen(acia_wen),
	.wdata(acia_wdata),
	.rdata(acia_rdata),
	.irq(acia_irq),
	.txd(uart_txd),
	.rxd(uart_rxd),
	.adiv_sel(2'b01)	// preset 1 (9600bps)
);

endmodule

// End of cyc2.v
